==> botanypots-fabric-1.21.1-21.1.12-sources.jar: net/darkhax/botanypots/common/impl/data/recipe/soil/BlockDerivedSoil.java <==
package net.darkhax.botanypots.common.impl.data.recipe.soil;

import com.mojang.serialization.Codec;
import com.mojang.serialization.MapCodec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import net.darkhax.bookshelf.common.api.util.DataHelper;
import net.darkhax.botanypots.common.api.data.display.types.Display;
import net.darkhax.botanypots.common.api.data.display.types.DisplayType;
import net.darkhax.botanypots.common.impl.Helpers;
import net.darkhax.botanypots.common.impl.data.display.types.BasicOptions;
import net.darkhax.botanypots.common.impl.data.display.types.SimpleDisplayState;
import net.minecraft.class_1792;
import net.minecraft.class_1802;
import net.minecraft.class_1856;
import net.minecraft.class_1865;
import net.minecraft.class_2248;
import net.minecraft.class_2350;
import net.minecraft.class_7923;
import net.minecraft.class_9129;
import net.minecraft.class_9139;
import java.util.Optional;
import java.util.Set;

public class BlockDerivedSoil extends BasicSoil {

    public static final MapCodec<BlockDerivedSoil> CODEC = BlockProperties.CODEC.xmap(BlockDerivedSoil::new, BlockDerivedSoil::getBlockProperties);
    public static final class_9139<class_9129, BlockDerivedSoil> STREAM = BlockProperties.STREAM.method_56432(BlockDerivedSoil::new, BlockDerivedSoil::getBlockProperties);
    public static final class_1865<BlockDerivedSoil> SERIALIZER = DataHelper.recipeSerializer(CODEC, STREAM);

    private final BlockProperties properties;

    public BlockDerivedSoil(BlockProperties properties) {
        super(properties.toBasic());
        this.properties = properties;
    }

    public BlockProperties getBlockProperties() {
        return this.properties;
    }

    public record BlockProperties(class_2248 block, Optional<class_1856> input, Optional<Display> display, float growthModifier, int lightLevel, Optional<BasicOptions> renderOptions) {
        public static final MapCodec<BlockProperties> CODEC = RecordCodecBuilder.mapCodec(instance -> instance.group(
                class_7923.field_41175.method_39673().fieldOf("block").forGetter(BlockProperties::block),
                class_1856.field_46095.optionalFieldOf("input").forGetter(BlockProperties::input),
                DisplayType.DISPLAY_STATE_CODEC.optionalFieldOf("display").forGetter(BlockProperties::display),
                Codec.FLOAT.optionalFieldOf("growth_modifier", 0f).forGetter(BlockProperties::growthModifier),
                Codec.INT.optionalFieldOf("light_level", 0).forGetter(BlockProperties::lightLevel),
                BasicOptions.CODEC.optionalFieldOf("render_options").forGetter(BlockProperties::renderOptions)
        ).apply(instance, BlockProperties::new));
        public static final class_9139<class_9129, BlockProperties> STREAM = class_9139.method_56437(
                (buf, props) -> {
                    Helpers.BLOCK_STREAM.encode(buf, props.block);
                    BasicSoil.Properties.STREAM.encode(buf, props.toBasic());
                },
                (buf) -> {
                    final class_2248 block = Helpers.BLOCK_STREAM.decode(buf);
                    final BasicSoil.Properties properties = BasicSoil.Properties.STREAM.decode(buf);
                    return fromBasic(block, properties);
                }
        );

        public BasicSoil.Properties toBasic() {
            final class_1856 input = this.input.orElseGet(() -> getSoil(this.block));
            final BasicOptions options = this.renderOptions.orElseGet(() -> BasicOptions.ofDefault(Set.of(class_2350.field_11036)));
            final Display display = this.display.orElseGet(() -> new SimpleDisplayState(this.block.method_9564(), options));
            return new BasicSoil.Properties(input, display, this.growthModifier, this.lightLevel);
        }

        public static BlockProperties fromBasic(class_2248 block, BasicSoil.Properties properties) {
            return new BlockProperties(block, Optional.of(properties.input()), Optional.of(properties.display()), properties.growthModifier(), properties.lightLevel(), Optional.empty());
        }

        private static class_1856 getSoil(class_2248 block) {
            final class_1792 blockItem = block.method_8389();
            if (blockItem != class_1802.field_8162) {
                return class_1856.method_8091(blockItem);
            }
            throw new IllegalStateException("Can not derive soil from block " + block + " id=" + class_7923.field_41175.method_10221(block));
        }
    }
}

==> botanypots-fabric-1.21.1-21.1.22-sources.jar: net/darkhax/botanypots/common/impl/data/recipe/soil/BlockDerivedSoil.java <==
package net.darkhax.botanypots.common.impl.data.recipe.soil;

import com.mojang.serialization.Codec;
import com.mojang.serialization.MapCodec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import net.darkhax.bookshelf.common.api.util.DataHelper;
import net.darkhax.botanypots.common.api.data.display.types.Display;
import net.darkhax.botanypots.common.api.data.display.types.DisplayType;
import net.darkhax.botanypots.common.impl.Helpers;
import net.darkhax.botanypots.common.impl.data.display.types.BasicOptions;
import net.darkhax.botanypots.common.impl.data.display.types.SimpleDisplayState;
import net.minecraft.class_1792;
import net.minecraft.class_1802;
import net.minecraft.class_1856;
import net.minecraft.class_1865;
import net.minecraft.class_2248;
import net.minecraft.class_2350;
import net.minecraft.class_7923;
import net.minecraft.class_9129;
import net.minecraft.class_9139;
import java.util.Optional;
import java.util.Set;

public class BlockDerivedSoil extends BasicSoil {

    public static final MapCodec<BlockDerivedSoil> CODEC = BlockProperties.CODEC.xmap(BlockDerivedSoil::new, BlockDerivedSoil::getBlockProperties);
    public static final class_9139<class_9129, BlockDerivedSoil> STREAM = BlockProperties.STREAM.method_56432(BlockDerivedSoil::new, BlockDerivedSoil::getBlockProperties);
    public static final class_1865<BlockDerivedSoil> SERIALIZER = DataHelper.recipeSerializer(CODEC, STREAM);

    private final BlockProperties properties;

    public BlockDerivedSoil(BlockProperties properties) {
        super(properties.toBasic());
        this.properties = properties;
    }

    public BlockProperties getBlockProperties() {
        return this.properties;
    }

    public record BlockProperties(class_2248 block, Optional<class_1856> input, Optional<Display> display, float growthModifier, int lightLevel, Optional<BasicOptions> renderOptions, float dropModifier) {
        public static final MapCodec<BlockProperties> CODEC = RecordCodecBuilder.mapCodec(instance -> instance.group(
                class_7923.field_41175.method_39673().fieldOf("block").forGetter(BlockProperties::block),
                class_1856.field_46095.optionalFieldOf("input").forGetter(BlockProperties::input),
                DisplayType.DISPLAY_STATE_CODEC.optionalFieldOf("display").forGetter(BlockProperties::display),
                Codec.FLOAT.optionalFieldOf("growth_modifier", 0f).forGetter(BlockProperties::growthModifier),
                Codec.INT.optionalFieldOf("light_level", 0).forGetter(BlockProperties::lightLevel),
                BasicOptions.CODEC.optionalFieldOf("render_options").forGetter(BlockProperties::renderOptions),
                Codec.floatRange(0f, Float.MAX_VALUE).optionalFieldOf("yield_modifier", 0f).forGetter(BlockProperties::dropModifier)
        ).apply(instance, BlockProperties::new));
        public static final class_9139<class_9129, BlockProperties> STREAM = class_9139.method_56437(
                (buf, props) -> {
                    Helpers.BLOCK_STREAM.encode(buf, props.block);
                    BasicSoil.Properties.STREAM.encode(buf, props.toBasic());
                },
                (buf) -> {
                    final class_2248 block = Helpers.BLOCK_STREAM.decode(buf);
                    final BasicSoil.Properties properties = BasicSoil.Properties.STREAM.decode(buf);
                    return fromBasic(block, properties);
                }
        );

        public BasicSoil.Properties toBasic() {
            final class_1856 input = this.input.orElseGet(() -> getSoil(this.block));
            final BasicOptions options = this.renderOptions.orElseGet(() -> BasicOptions.ofDefault(Set.of(class_2350.field_11036)));
            final Display display = this.display.orElseGet(() -> new SimpleDisplayState(this.block.method_9564(), options));
            return new BasicSoil.Properties(input, display, this.growthModifier, this.lightLevel, this.dropModifier);
        }

        public static BlockProperties fromBasic(class_2248 block, BasicSoil.Properties properties) {
            return new BlockProperties(block, Optional.of(properties.input()), Optional.of(properties.display()), properties.growthModifier(), properties.lightLevel(), Optional.empty(), properties.yieldModifier());
        }

        private static class_1856 getSoil(class_2248 block) {
            final class_1792 blockItem = block.method_8389();
            if (blockItem != class_1802.field_8162) {
                return class_1856.method_8091(blockItem);
            }
            throw new IllegalStateException("Can not derive soil from block " + block + " id=" + class_7923.field_41175.method_10221(block));
        }
    }
}
